import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import { CommandHandler } from '@nestjs/cqrs';
import { Messager } from '../../adapters/secondaries/messager';
import { UsersRepository } from '../../adapters/secondaries/users.repository';
import { CreateUserCommand } from '../../commands/create-user.command';
import { CreateUserRequest } from '../dtos/create-user.request';
import { User } from '../entities/user';

@CommandHandler(CreateUserCommand)
export class CreateUserUseCase {
  constructor(
    private readonly repository: UsersRepository,
    private readonly messager: Messager,
    @InjectMapper() private readonly mapper: Mapper,
  ) {}

  execute = async (command: CreateUserCommand): Promise<User> => {
    const entity = this.mapper.map(
      command.createUserRequest,
      CreateUserRequest,
      User,
    );

    try {
      const user = await this.repository.create(entity);
      this.messager.publish('user.create', JSON.stringify(user));
      this.messager.publish('logging.user.create.info', JSON.stringify(user));
      return user;
    } catch (error) {
      let key = 'logging.user.create.crit';
      if (error.message.includes('Already exists')) {
        key = 'logging.user.create.warning';
      }
      this.messager.publish(
        key,
        JSON.stringify({
          command,
          error,
        }),
      );
      throw error;
    }
  };
}
